-------------------------------------------
--- Author: Ketho (EU-Boulderfist)		---
--- License: Public Domain				---
-------------------------------------------
-- http://wow.curse.com/downloads/wow-addons/details/simpleding.aspx
-- http://www.wowinterface.com/downloads/info19479-SimpleDing.html

SimpleDing = LibStub("AceAddon-3.0"):NewAddon("SimpleDing", "AceEvent-3.0", "AceTimer-3.0", "AceConsole-3.0")
local SD = SimpleDing

-- All comments can be found in ReadySetDing,
-- as this is just the naked/minimalistic version of it ...
-- This was actually quite an useful experience,
-- since I revisited the core part and made it "simpler"

local gsub, time = gsub, time
local GetGuildRosterInfo = GetGuildRosterInfo
local SCM = SendChatMessage

-- I don't want the capitalized GlobalStrings (except for German locale)
local D_SECONDS = strlower(D_SECONDS)
local D_MINUTES = strlower(D_MINUTES)
local D_HOURS = strlower(D_HOURS)
local D_DAYS = strlower(D_DAYS)
local D_MINUTES2 = gsub(D_MINUTES, "%%d", "%%.1f")

if GetLocale() == "deDE" then
	D_SECONDS = _G.D_SECONDS
	D_MINUTES = _G.D_MINUTES
	D_HOURS = _G.D_HOURS
	D_DAYS = _G.D_DAYS
end

local playerLevel = UnitLevel("player")
local lastPlayed = time()

local profile, char
local TPM_totalTime, TPM_currentTime = 0, 0
local TPM_totalTime2, TPM_currentTime2
local levelTime, levelTime2
local playerDinged

local guild = {}
local bttn = CreateFrame("Button")
local defaults = {profile = {dingMessage = "Ding! Level [LEVEL] in [TIME]"}}
local options = {
	type = "group",
	name = "Simple|cffFFFFFFDing|r",
	args = {
		Screenshot = {
			type = "toggle",
			order = 1,
			descStyle = "",
			name = "|TInterface\\Icons\\inv_misc_spyglass_03:20:20:1:0:64:64:4:60:4:60|t  "..BINDING_NAME_SCREENSHOT,
			get = function() return profile.Screenshot end,
			set = function(i, v) profile.Screenshot = v end,
		},
		newline1 = {type = "description", order = 2, name = ""},
		GuildAnnounce = {
			type = "toggle",
			order = 3,
			descStyle = "",
			name = "|TInterface\\Icons\\Ability_Warrior_RallyingCry:20:20:1:0:64:64:4:60:4:60|t  |cff40FF40"..GUILD.."|r "..CHAT_ANNOUNCE,
			get = function() return profile.GuildAnnounce end,
			set = function(i, v) profile.GuildAnnounce = v end,
		},
		newline2 = {type = "description", order = 4, name = ""},
		GuildMemberDings = {
			type = "toggle",
			order = 5,
			descStyle = "",
			name = "|TInterface\\GuildFrame\\GuildLogo-NoLogo:20:20:1:0:64:64:14:51:14:51|t  "..GUILD.." Dings",
			get = function(i) return profile.GuildMemberDings end,
			set = function(i, v) profile.GuildMemberDings = v
				if IsInGuild() then
					if v then
						SD.GuildRosterTimer = SD:ScheduleRepeatingTimer(function() GuildRoster() end, 11)
					else
						SD:CancelTimer(SD.GuildRosterTimer)
					end
				end
			end,
		},
		DingMessage = {
			type = "input",
			order = 6,
			width = "full",
			name = " ",
			usage = "\n|cffADFF2F[LEVEL]|r |cffFFFFFF= New Level|r\n|cff71D5FF[TIME]|r |cffFFFFFF= Level Time|r\n|cff71D5FF[TOTAL]|r |cffFFFFFF= Total Time|r",
			get = function(i) return profile.dingMessage end,
			set = function(i, v) profile.dingMessage = v
				if #strtrim(v) == 0 then profile.dingMessage = defaults.profile.dingMessage end
			end,
		},
		Example = {
			type = "description",
			order = 7,
			name = function() return "   "..SD:ReplaceText(profile.dingMessage, true) end,
		},
	},
}

function SD:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("SimpleDingDB", defaults, true)
	profile, char = self.db.profile, self.db.char
	LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable("SimpleDing", options)
	SD.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("SimpleDing", "Simple|cffFFFFFFDing|r")
	
	self:RegisterChatCommand("sd", "SlashCmd")
	self:RegisterChatCommand("simpleding", "SlashCmd")

	self.db.global.version = 0.2	
	char.levelTime = char.levelTime or {}
	char.totalTime = char.totalTime or {}
end

function SD:OnEnable()
	self:RegisterEvent("PLAYER_LEVEL_UP")
	self:RegisterEvent("TIME_PLAYED_MSG")
	self:RegisterEvent("GUILD_ROSTER_UPDATE")

	self:ScheduleTimer(function() if TPM_totalTime == 0 then RequestTimePlayed() end end, 5)
	if profile.GuildMemberDings then
		self.GuildRosterTimer = self:ScheduleRepeatingTimer(function() GuildRoster() end, 11)
	end
end

-- it's not like this is going to be used manually, or from other addons, but whatever
function SD:OnDisable()
	self:UnregisterAllEvents()
	self:CancelAllTimers()
end

function SD:SlashCmd()
	InterfaceOptionsFrame_OpenToCategory(SD.optionsFrame)
end

local function TimetoString(value)
	local seconds = mod(floor(value), 60)
	local minutes = mod(floor(value/60), 60)
	local hours = mod(floor(value/3600), 24)
	local days = floor(value/86400)

	-- need to get rid of the pipe | characters
	-- so they can be used in SendChatMEssage()
	local fseconds = bttn:GetText(bttn:SetFormattedText(D_SECONDS, seconds))
	local fminutes = bttn:GetText(bttn:SetFormattedText(D_MINUTES, minutes))
	local fhours = bttn:GetText(bttn:SetFormattedText(D_HOURS, hours))
	local fdays = bttn:GetText(bttn:SetFormattedText(D_DAYS, days))

	if value >= 0 and value < 60 then
		return fseconds
	elseif value >= 60 and value < 3600 then
		-- bit ugly
		if seconds >= 6 then
			return bttn:GetText(bttn:SetFormattedText(D_MINUTES2, tonumber(minutes.."."..format("%.0f",floor(seconds/6)))))
		else
			return fminutes
		end
	elseif value >= 3600 and value < 86400 then
		if minutes > 0 then
			return fhours..", "..fminutes
		else
			return fhours
		end
	elseif value >= 86400 then
		if hours > 0 then
			return fdays..", "..fhours
		else
			return fdays
		end
	end
end

local function AddedTime()
	return time() - lastPlayed
end

local function GetClassColor(class)
	return format("%02X%02X%02X", RAID_CLASS_COLORS[class].r*255, RAID_CLASS_COLORS[class].g*255, RAID_CLASS_COLORS[class].b*255)
end

function SD:ReplaceText(msg, example)
	local gsub = gsub

	if example then
		msg = gsub(msg, "%[[Ll][Ee][Vv][Ee][Ll]%]", "|cffADFF2F"..playerLevel+1 .."|r")
		msg = gsub(msg, "%[[Tt][Ii][Mm][Ee]%]", "|cff71D5FF"..TimetoString(TPM_currentTime+AddedTime()).."|r")
		msg = gsub(msg, "%[[Tt][Oo][Tt][Aa][Ll]%]", "|cff71D5FF"..TimetoString(TPM_totalTime+AddedTime()).."|r")
	else
		msg = gsub(msg, "%[[Ll][Ee][Vv][Ee][Ll]%]", playerLevel)
		msg = gsub(msg, "%[[Tt][Ii][Mm][Ee]%]", TimetoString(levelTime2))
		msg = gsub(msg, "%[[Tt][Oo][Tt][Aa][Ll]%]", TimetoString(TPM_totalTime))
	end
	return msg
end

function SD:PLAYER_LEVEL_UP(_, level)
	playerLevel, playerDinged = level, true
	RequestTimePlayed()
end

function SD:TIME_PLAYED_MSG(_, ...)
	TPM_totalTime, TPM_currentTime = ...
	lastPlayed = time()

	if playerDinged then
		playerDinged = false
		
		local a = char.totalTime[playerLevel-1]
		if a then
			levelTime2 = TPM_totalTime - a
		else
			-- no previous level data, fall back to less accurate data
			levelTime2 = TPM_currentTime2 + (TPM_totalTime-TPM_totalTime2)
		end

		char.levelTime[playerLevel] = levelTime2
		char.totalTime[playerLevel] = TPM_totalTime

		-- announce stuff
		local text = SD:ReplaceText(profile.dingMessage)
		SCM(text, GetNumPartyMembers() > 0 and "PARTY" or "SAY")
		if profile.GuildAnnounce and IsInGuild() then
			SCM(text, "GUILD")
		end

		if profile.Screenshot then
			self:ScheduleTimer(function() Screenshot() end, 1)
		end
	end
	-- update/prepare stuff for next levelup
	TPM_totalTime2, TPM_currentTime2 = TPM_totalTime, TPM_currentTime
	LibStub("AceConfigRegistry-3.0"):NotifyChange("SimpleDing")
end

function SD:GUILD_ROSTER_UPDATE()
	if profile.GuildAnnounce then
		for i = 1, GetNumGuildMembers() do
			local name, _, _, level, _, _, _, _, _, _, classFileName = GetGuildRosterInfo(i)
			-- sanity check: sometimes GetGuildRosterInfo() only returns a nil
			if name and guild[name] and level > guild[name] then
				RaidNotice_AddMessage(RaidWarningFrame, "|cff"..GetClassColor(classFileName)..name.."|r dinged "..LEVEL.." |cffADFF2F"..level.."|r", {r=1, g=1, b=1})
			end
			guild[name] = level
		end
	end
end